# Generated by Django 1.11.3 on 2017-07-24 11:44

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("mail", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="queuedmail",
            name="sent",
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
