variables()
// normalize-css()
// base()
// typography()
components()
scrollbars()

a
	color: var(--pretalx-clr-primary)
	text-decoration: none

html, body
	margin: 0
	--pretalx-clr-primary: #673ab7

.bunt-scrollbar
	min-height: 0 // firefox can fuck off into space
